/*
 * lstime - a directory listing function with time stamp
 *
 * Copyright (c) 2003 Paul H. Liu <paul@thev.net>
 * 
 *  See the file `COPYING' for copyright notice.
 */

#include <stdio.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define MAXLEN 4096

char filename[MAXLEN];

struct tuple {
    char filename[MAXLEN];
    time_t mtime;
};

void insert(struct tuple **list, int *size, int *n, char *filename, time_t mtime)
{
    if (*n == *size)
    {
        struct tuple *tmp;
        tmp = (struct tuple *)malloc(sizeof(struct tuple) * (*size) * 2);
        memcpy(tmp, *list, sizeof(struct tuple) * (*size));
        free(*list);
        *list = tmp;
        *size = *size * 2;
    }
    strcpy((*list)[*n].filename, filename);
    (*list)[*n].mtime = mtime;
    *n = *n + 1;
}

int main(int argc, char** argv)
{
    struct stat buf;
    struct dirent *dirs;
    DIR *dir;
    char *dirname;
    int time, i, j, r, n, size = 500;
    struct tuple *list;
    
    if (argc <= 1)
    {
        dirname = ".";
        argc = 2;
    }
    else 
    {
        dirname = argv[1];
    }

    time = 0;

    n = 0;
    list = (struct tuple *)malloc(sizeof(struct tuple) * size);

    for (i = 1; i < argc; i++)
    {
	if (strlen(dirname) > 0 && dirname[0] == '-')
	{
	    if (index(dirname, 't')) time = 1;
	}
	else
	{
	    r = stat(dirname, &buf);

	    if (!r && S_ISDIR (buf.st_mode))
	    {
	        dir = opendir (dirname);
                while (dir && (dirs = readdir (dir)) != 0)
                {
                    if (!strcmp (dirs->d_name, "."))
                        continue;
                    if (!strcmp (dirs->d_name, ".."))
                        continue;
                    sprintf(filename, "%s/%s", dirname, dirs->d_name);
                    r = stat(filename, &buf);
                    if (!r)
                    {
                    	insert(&list, &size, &n, dirs->d_name, buf.st_mtime);
                    }
                }
                dir && closedir (dir);
            }
            else if (!r)
            {
            	insert(&list, &size, &n, dirname, buf.st_mtime);
            }
        }
        if (i < argc - 1) dirname = argv[i + 1];    	
    }
    
    for (i = 0; i < n - 1; i++)
        for (j = i + 1; j < n; j++)
        {
            if ((time && (list[i].mtime < list[j].mtime)) ||
            	(!time && (strcmp(list[i].filename, list[j].filename) > 0)))
            {
                struct tuple tmp;
                tmp = list[i];
                list[i] = list[j];
                list[j] = tmp;
            }
        }
    
    for (i = 0; i < n; i++)
        printf("%s\t%d\n", list[i].filename, list[i].mtime);

    free(list);

    return 0;           
}
